const gulp = require('gulp');
const zip = require('gulp-zip');
const watch = require('gulp-watch');
const rename = require("gulp-rename");
const gulpif = require('gulp-if');
const toObject = require("gulp-properties-to-object");
const util = require('gulp-util');
const replace = require('gulp-token-replace');

var properties = {};

// strips whitespace from property values
// adds timestamp and date properties
function normalize(object) {
    for (var property in object) {
        object[property] = object[property].trim ? object[property].trim() : object[property];
    }
    var now = new Date();
    object.TIMESTAMP = Math.floor(now.getTime());
    object.TODAY = now.toISOString().slice(0, 10);
}

// 'new' task - generates a new component from the template folder
// run by 'gulp new [-templateName]'
// if no -templateName argument supplied, 'default' is used
// library.properties is used to populate .library-config.xml
// new-component.properties is used to populate component files
//  - edit this file before running the 'new' task
//  - existing output files will be overwritten without warning
gulp.task('new', () => {
    gulp.src("*.properties").pipe(toObject(properties)).on('finish', function () {
        normalize(properties);
        let templateName = process.argv[3] || '-default';
        util.log('Generating new component using template: ' + templateName);

        gulp.src('template/' + templateName.substring(1) + '/*.*', { dot: true })
            .pipe(gulpif(function (file) { return !file.path.endsWith('.png') }, replace({ global: properties, prefix:'@@', suffix:'@@' })))
            .pipe(rename(function (path) {
                var componentName = properties['component_filename'];
                if (!path.extname.endsWith('xml')) {
                    path.dirname += "/widgets/" + componentName;
                }
                path.basename = path.basename.replace('template', componentName);
                util.log('Processed: ' + path.dirname + '/' + path.basename);
            }))
            .pipe(gulp.dest('src'));
    });
});

// 'zip' task - packages 'src' folder hierarchy into a Maestro library zip file
gulp.task('zip', () =>
    gulp.src("*.properties").pipe(toObject(properties)).on('finish', function () {
        normalize(properties);
        gulp.src('src/**/*', { dot: true })
            .pipe(zip('maestro-lib-' + properties['library_code'] + '.zip'))
            .pipe(gulp.dest('dist'));
    })
);

// 'watch' task - watches folder hierarchy for changes and runs a task when changes are detected
// watches the src folder hierarchy and runs 'zip' task
// watches new-component.properties file and runs 'new' task
gulp.task('watch', function () {
    gulp.watch('src/**/*', ['zip']);
    gulp.watch('new-component.properties', ['new']);
});

// creates a redistributable package with all templates
// package can be installed by installing nodejs, unzip package to a folder, run 'npm install', then run gulp new [-templateName]
gulp.task('redist', () =>
    gulp.src(['gulpfile.js', 'library.properties', 'new-component.properties', 'package.json', 'template/**'], { base: '.', dot: true }, { dot: true })
        .pipe(zip('avoka-native-component-tools.zip'))
        .pipe(gulp.dest('.'))
);
